unit MainFormUnit;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ExtCtrls, Menus, ComCtrls, ExtDlgs, JPEG, Buttons, Clipbrd,
  IniFiles, ProgressFormUnit;

const
  MaxWaveletSize=32;        //Taille maximale de la fentre d'ondelettes

type
  TWaveletData=array[0..2*MaxWaveletSize-1,0..MaxWaveletSize-1] of Single;       //Transforme en ondelettes d'un rectangle d'image

  TWeightMatrix=array[0..1,0..1] of Single;       //Matrice de pondration 2 x 2
  PWeightMatrix=^TWeightMatrix;

  TGlyphData=record                         //Les donnes associes  un caractre d'une police: taille + transforme en ondelettes
    Width,Height:Byte;
    WaveletData:TWaveLetData;
    WeightsDomain:TWeightMatrix;
  end;
  PGlyphData=^TGlyphData;

  TMainForm = class(TForm)
    PageControl: TPageControl;
    OriginalImageTabSheet: TTabSheet;
    ASCIIConversionTabSheet: TTabSheet;
    OriginalImageScrollBox: TScrollBox;
    OriginalImagePaintBox: TPaintBox;
    ASCIIMemo: TMemo;
    OpenPictureDialog: TOpenPictureDialog;
    FontDialog: TFontDialog;
    SidePanel2: TPanel;
    FontPanel: TPanel;
    ChangeFontSpeedButton: TSpeedButton;
    UseGammaCorrectionCheckBox: TCheckBox;
    GroupBox1: TGroupBox;
    LoadMatrixSpeedButton: TSpeedButton;
    SaveNewMatrixSpeedButton: TSpeedButton;
    DeleteMatrixSpeedButton: TSpeedButton;
    MatrixPanel1: TPanel;
    MatrixTrackBar1: TTrackBar;
    MatrixPanel2: TPanel;
    MatrixTrackBar2: TTrackBar;
    MatrixPanel3: TPanel;
    MatrixTrackBar3: TTrackBar;
    MatrixPanel4: TPanel;
    MatrixTrackBar4: TTrackBar;
    MatrixListBox: TListBox;
    MatrixNameEdit: TEdit;
    SymmetricCheckBox: TCheckBox;
    OnlyAlphanumericCheckBox: TCheckBox;
    WhiteOnBlackCheckBox: TCheckBox;
    FilteredImageTabSheet: TTabSheet;
    FilteredImageScrollBox: TScrollBox;
    FilteredImagePaintBox: TPaintBox;
    SidePanel1: TPanel;
    MedianFilterSizeRadioGroup: TRadioGroup;
    WaveletQualityRadioGroup: TRadioGroup;
    GroupBox2: TGroupBox;
    KeepRatioCheckBox: TCheckBox;
    CopyToClipboardSpeedButton: TSpeedButton;
    CopyHTMLToClipboardSpeedButton: TSpeedButton;
    BottomPanel: TPanel;
    BackBitBtn: TBitBtn;
    NextBitBtn: TBitBtn;
    ProcessBitBtn: TBitBtn;
    XFactorTrackBar: TTrackBar;
    YFactorTrackBar: TTrackBar;
    StaticPanel1: TPanel;
    StaticPanel2: TPanel;
    ResizePanel: TPanel;
    RatioSpeedButton1: TSpeedButton;
    RatioSpeedButton2: TSpeedButton;
    RatioSpeedButton3: TSpeedButton;
    RatioSpeedButton4: TSpeedButton;
    RatioSpeedButton5: TSpeedButton;
    ResizingMethodRadioGroup: TRadioGroup;
    AutoAdjustCheckBox: TCheckBox;
    RidgeletModeCheckBox: TCheckBox;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure OriginalImagePaintBoxPaint(Sender: TObject);
    procedure ChangeFontSpeedButtonClick(Sender: TObject);
    procedure CopyToClipboardSpeedButtonClick(Sender: TObject);
    procedure CopyHTMLToClipboardSpeedButtonClick(Sender: TObject);
    procedure MatrixTrackBar1Change(Sender: TObject);
    procedure MatrixListBoxClick(Sender: TObject);
    procedure MatrixNameEditChange(Sender: TObject);
    procedure MatrixListBoxDblClick(Sender: TObject);
    procedure SaveNewMatrixSpeedButtonClick(Sender: TObject);
    procedure DeleteMatrixSpeedButtonClick(Sender: TObject);
    procedure FilteredImagePaintBoxPaint(Sender: TObject);
    procedure FontDialogApply(Sender: TObject; Wnd: HWND);
    procedure WaveletQualityRadioGroupClick(Sender: TObject);
    procedure PageControlChange(Sender: TObject);
    procedure NextBitBtnClick(Sender: TObject);
    procedure BackBitBtnClick(Sender: TObject);
    procedure ProcessBitBtnClick(Sender: TObject);
    procedure XFactorTrackBarChange(Sender: TObject);
    procedure RatioSpeedButton1Click(Sender: TObject);
    procedure AutoAdjustCheckBoxClick(Sender: TObject);
  private
    FOriginalBitmap,FFilteredBitmap,FRasterBitmap:TBitmap;           //les diffrents bitmaps de travail: original=image originale, filtered=image filtre et raster=bitmap de travail pour la transforme en ondelettes
    FGlyphsData:array[#30..#255] of TGlyphData;        //La transforme en ondelettes pour les caractres de la police courante du 30me au dernier
    FGrayScaleLogPalette:PLogPalette;       //la palette pour les conversions en niveaux de gris
    FWeightMatrix:TWeightMatrix;       //la matrice de pondration courante
    FWaveletSize:Integer;       //la taille courante de la fentre d'ondelettes

    function MakeWaveletData(p:TBitmap;x,y,w,h:Integer;Inverted:Boolean):TWaveLetData;   //calcule la transforme en ondelettes d'un bitmap dans le rectangle (x,y,x+w,y+h)
  public
    procedure IniLoad;    //lecture du fichier ini
    procedure IniSave;    //sauvegarde du fichier ini

    procedure Applyfont;   //mise  jour de l'interface lorsqu'on change la police
    procedure ApplyResize;    //mise  jour de l'interface lorsqu'on change les paramtres de redimensionnement

    procedure LoadImage;              //Ouvre la boite de dialogue d'ouverture de fichier image
    procedure BuildFilteredBitmap;    //Construit l'image redimensionne et filtre
    procedure MakeFontGlyphs;         //Construit le tableau des donnes de la police courante
    procedure ProcessASCIIConversion; //Effectue la conversion de l'image filtre en ASCII
  end;

var
  MainForm: TMainForm;

implementation

uses Math;

{$R *.dfm}

procedure TMainForm.FormCreate(Sender: TObject);     //au moment de la cration de la form
var
  a:Integer;
begin
  WaveletQualityRadioGroupClick(nil);           //mise  jour de l'interface
  MatrixTrackBar1.Tag:=Integer(MatrixPanel1);   //Chacune des 4 trackbars de la matrice contient dans sa proprit tag le panel associ pour afficher sa valeur.
  MatrixTrackBar2.Tag:=Integer(MatrixPanel2);   //Voir en particulier l'vnement OnChange de ces 4 trackbars: MatrixTrackBar1Change
  MatrixTrackBar3.Tag:=Integer(MatrixPanel3);
  MatrixTrackBar4.Tag:=Integer(MatrixPanel4);
  MatrixTrackBar1Change(MatrixTrackBar1);       //mise  jour de l'interface, voir remarque prcdente
  MatrixTrackBar1Change(MatrixTrackBar2);
  MatrixTrackBar1Change(MatrixTrackBar3);
  MatrixTrackBar1Change(MatrixTrackBar4);
  ChangeFontSpeedButton.Align:=alRight;         //mise  jour de l'interface
  GetMem(FGrayScaleLogPalette,SizeOf(TLogPalette)+255*SizeOf(TPaletteEntry));   //On alloue la mmoire pour la palette
  {$R-}
  for a:=0 to 255 do begin                                                      //On construit les 256 entres de la palette
    FGrayScaleLogPalette.palPalEntry[a].peRed:=a;
    FGrayScaleLogPalette.palPalEntry[a].peGreen:=a;
    FGrayScaleLogPalette.palPalEntry[a].peBlue:=a;
    FGrayScaleLogPalette.palPalEntry[a].peFlags:=PC_NOCOLLAPSE;
  end;
  {$R+}
  FGrayScaleLogPalette.palVersion:=$300;
  FGrayScaleLogPalette.palNumEntries:=256;
  FOriginalBitmap:=TBitmap.Create;       //Bitmap original
  FFilteredBitmap:=TBitmap.Create;       //Bitmap filtr
  FFilteredBitmap.PixelFormat:=pf8bit;   //passage en 8 bits
  FFilteredBitmap.Palette:=CreatePalette(FGrayScaleLogPalette^);  //passage en niveaux de gris
  FRasterBitmap:=TBitmap.Create;         //Bitmap de travail
  FRasterBitmap.PixelFormat:=pf8bit;     //idem: passage en niveaux de gris
  FRasterBitmap.Palette:=CreatePalette(FGrayScaleLogPalette^);
  Applyfont;                   //mise  jour de l'interface
  IniLoad;                     //Lecture du fichier ini
  PageControlChange(nil);      //mise  jour de l'interface
  AutoAdjustCheckBoxClick(nil);
end;

procedure TMainForm.FormDestroy(Sender: TObject);
begin
  IniSave;                     //On sauvegarde le fichier ini
  FFilteredBitmap.Destroy;     //On libre la mmoire...
  FRasterBitmap.Destroy;
  FOriginalBitmap.Destroy;
  FreeMem(FGrayScaleLogPalette);
end;

procedure TMainForm.OriginalImagePaintBoxPaint(Sender: TObject);   //Pour que la paintbox affiche l'image originale...
begin
  OriginalImagePaintBox.Canvas.Draw(0,0,FOriginalBitmap);
end;

procedure TMainForm.MakeFontGlyphs;       //On cre la transforme en ondelettes des caractres 30  255 de la police courante
var
  c:Char;
  p:TBitmap;
  a,i,j:Integer;
  s,t:Single;

  procedure RecBuildGlyphWaveletDomain(var g:TGlyphData);         //Cette fonction construit la matrice de rpartition de l'ondelette dans les diffrentes directions
  var                                                             //(horizontal, vertical et diagonal slope) de faon  pouvoir dterminer automatiquement
    a,b,i:Integer;                                                //la matrice la mieux adapte  la police slectionne lorsqu'on utilise le mode "auto-adjust".
    r:Single;
    w:TWeightMatrix;
  begin
    ZeroMemory(@g.WeightsDomain,SizeOf(TWeightMatrix));
    g.WeightsDomain[0,0]:=g.WaveletData[0,0];
    r:=(1-0.25)/(1-Exp(Ln(0.25)*Ln(FWaveLetSize)/Ln(2)));           //constante de renormalisation
    i:=1;
    while i<FWaveletSize do begin                 //on calcule une norme sur le paquet d'ondelettes pour avoir au final le "pouvoir discriminant" du caractre
      ZeroMemory(@w,SizeOf(TWeightMatrix));       //dans les diffrentes directions
      for a:=0 to i-1 do
        for b:=0 to i-1 do begin
          w[0,1]:=w[0,1]+g.WaveletData[a,b+i];
          w[1,0]:=w[1,0]+g.WaveletData[a+i,b];
          w[1,1]:=w[1,1]+g.WaveletData[a+i,b+i];
        end;
      g.WeightsDomain[0,1]:=g.WeightsDomain[0,1]+Abs(w[0,1])*r;
      g.WeightsDomain[1,0]:=g.WeightsDomain[1,0]+Abs(w[1,0])*r;
      g.WeightsDomain[1,1]:=g.WeightsDomain[1,1]+Abs(w[1,1])*r;
      r:=r*0.25;
      i:=i*2;
    end;
  end;

begin
  p:=TBitmap.Create;                        //bitmap de travail pour dessiner les caractres
  p.Canvas.Font.Assign(FontDialog.Font);    //on rcupre la police courante
  a:=0;
  s:=0;
  t:=1E10;
  for c:=#30 to #255 do begin
    if a<p.Canvas.TextHeight(c) then      // la fin de la boucle a contiendra la plus grande hauteur de tous les caractres
      a:=p.Canvas.TextHeight(c);
  end;
  p.Height:=a;
  for c:=#30 to #255 do begin
    if OnlyAlphanumericCheckBox.Checked and not IsCharAlphaNumeric(c) then    //Si on est en mode "use only alphanumeric characters" on ignore les caractres non alpha-numriques
      Continue;
    p.Width:=p.Canvas.TextWidth(c);    //on adapte les dimensions au caractre
    p.Height:=a;
    p.Canvas.TextOut(0,0,c);           //on dessine le caractre
    FGlyphsData[c].Width:=p.Width;
    FGlyphsData[c].Height:=a;
    FGlyphsData[c].WaveletData:=MakeWaveletData(p,0,0,p.Width,p.Height,False);    //on calcule la transforme en ondelettes pour le caractre 
    if s<FGlyphsData[c].WaveletData[0,0] then  // la fin de la boucle s contiendra la plus grande intensit moyenne de tous les caractres
      s:=FGlyphsData[c].WaveletData[0,0];
    if t>FGlyphsData[c].WaveletData[0,0] then  // la fin de la boucle t contiendra la plus petite intensit moyenne de tous les caractres (zro si le caractre "espace" existe)
      t:=FGlyphsData[c].WaveletData[0,0];
  end;
  for c:=#30 to #255 do begin
    if OnlyAlphanumericCheckBox.Checked and not IsCharAlphaNumeric(c) then    //On ignore les caractres non alpha-numriques si spcifi par l'utilisateur
      Continue;
    if UseGammaCorrectionCheckBox.Checked and (s<>t) then begin               //Correction du gamma
      for i:=0 to FWaveletSize-1 do  // la fin de la boucle, le caractre le moins fonc aura une intensit moyenne de 255, et le plus fonc de zro
        for j:=0 to FWaveletSize-1 do
          if i+j=0 then
            FGlyphsData[c].WaveletData[i,j]:=(FGlyphsData[c].WaveletData[i,j]-t)*255/(s-t)   //le premier coefficient de la transforme en ondelette (l'intensit moyenne) est
          else                                                                               //affect non linairement
            FGlyphsData[c].WaveletData[i,j]:=FGlyphsData[c].WaveletData[i,j]*255/s;    //les autres coefficients sont affects linairement (par construction de l'ondelette)
    end;
    RecBuildGlyphWaveletDomain(FGlyphsData[c]);
  end;
  p.Destroy;     //on libre la mmoire
end;

function TMainForm.MakeWaveletData(p: TBitmap; x, y, w,   //cette fonction calcule la transforme en ondelettes du bitmap en niveaux de gris p dans le rectangle (x,y,x+w,y+h)
  h: Integer; Inverted: Boolean): TWaveLetData;           //le paramtre inverted permet de calculer la transforme en ondelettes du ngatif de p.

  procedure RecMake2DWavelet(var w:TWaveLetData;n:Byte);   //procdure rcursive de calcul de filtrage en paquets ondelettes
  var
    v:TWaveLetData;
    a,b:Integer;
  begin
    n:=n div 2;

    for a:=0 to 2*n-1 do
      for b:=0 to 2*n-1 do
        v[a,b]:=w[a,b];
    for b:=0 to 2*n-1 do
      for a:=0 to n-1 do begin
        w[a,b]:=0.5*(v[2*a,b]+v[2*a+1,b]);         //filtre passe-bas vertical
        w[n+a,b]:=0.5*(v[2*a,b]-v[2*a+1,b]);       //filtre passe-haut vertical
      end;

    for a:=0 to 2*n-1 do
      for b:=0 to 2*n-1 do
        v[a,b]:=w[a,b];
    for a:=0 to 2*n-1 do
      for b:=0 to n-1 do begin
        w[a,b]:=0.5*(v[a,2*b]+v[a,2*b+1]);        //filtre passe-bas horizontal
        w[a,n+b]:=0.5*(v[a,2*b]-v[a,2*b+1]);      //filtre passe-haut horizontal
      end;
      
    if n>1 then
      RecMake2DWavelet(w,n);                        //on recommence sur les basses frquences
  end;

  procedure RecMake1DWaveLet(var w:array of Single;n:Integer);     //Calcule une transforme en ondelettes unidimensionnelle
  var
    v:array[0..2*MaxWaveletSize-1] of Single;
    a:Integer;
  begin
    n:=n div 2;

    for a:=0 to 2*n-1 do
      v[a]:=w[a];
    for a:=0 to n-1 do begin
      w[a]:=0.5*(v[2*a]+v[2*a+1]);            //filtre passe-haut
      w[a+n]:=0.5*(v[2*a]-v[2*a+1]);          //filtre passe-bas
    end;
  end;

  function NumberOfBitsNeeded(PowerOfTwo : Integer) : Integer;
  var
    I : Integer;
  begin
    Result:=0;
    for I := 0 to {MaxPower}16 do
      begin
        if (PowerOfTwo and (1 shl I)) <> 0 then
          begin
            NumberOfBitsNeeded := I;
            Exit;
          end;
      end;
  end;

  function ReverseBits(Index, NumBits : Integer) : Integer;
  var
    I, Rev : Integer;
  begin
    Rev := 0;
    for I := 0 to NumBits - 1 do
      begin
        Rev := (Rev shl 1) or (Index and 1);
        Index := Index shr 1;
      end;
    ReverseBits := Rev;
  end;

{  procedure FourierTransform(const RealIn,ImagIn:array of Single;var RealOut,ImagOut:array of Single;Theta:Single;NumSamples : Integer);
  var
    NumBits, I, J, K, N, BlockSize, BlockEnd : Integer;
    Delta_angle, Delta_ar : Single;
    Alpha, Beta : Single;
    Tr, Ti, Ar, Ai : Single;
//    RealOut,ImagOut:array[0..MaxWaveletSize-1] of Single;
  begin
    NumBits := NumberOfBitsNeeded(NumSamples);
    for I := 0 to NumSamples - 1 do
      begin
        J := ReverseBits(I, NumBits);
        RealOut[J] := RealIn[I];
        ImagOut[J] := ImagIn[I];
      end;

    BlockEnd := 1;
    BlockSize := 2;
    while BlockSize <= NumSamples do
      begin
        Delta_angle := Theta / BlockSize;
        Alpha := Sin(0.5 * Delta_angle);
        Alpha := 2.0 * Alpha * Alpha;
        Beta := Sin(Delta_angle);

        I := 0;
        while I < NumSamples do 
          begin
            Ar := 1.0;    (* cos(0) *)
            Ai := 0.0;    (* sin(0) *)

            J := I;
            for N := 0 to BlockEnd - 1 do
              begin
                K := J + BlockEnd;
                Tr := Ar * RealOut[K] - Ai * ImagOut[K];
                Ti := Ar * ImagOut[K] + Ai * RealOut[K];
                RealOut[K] := RealOut[J] - Tr;
                ImagOut[K] := ImagOut[J] - Ti;
                RealOut[J] := RealOut[J] + Tr;
                ImagOut[J] := ImagOut[J] + Ti;
                Delta_ar := Alpha * Ar + Beta * Ai;
                Ai := Ai - (Alpha * Ai - Beta * Ar);
                Ar := Ar - Delta_ar;
                Inc(J);
              end;

            I := I + BlockSize;
          end;

        BlockEnd := BlockSize;
        BlockSize := BlockSize shl 1;
      end;
    //for i:=0 to NumSamples-1 do
    //  RealIn[(i+(NumSamples div 2)) mod NumSamples]:=Sqrt((Sqr(RealOut[i])+Sqr(ImagOut[i])))///Sqrt(NumSamples);
  end; }

  procedure FourierTransform(const RealIn,ImagIn:array of Single;var RealOut,ImagOut:array of Single;Theta:Single;NumSamples:Integer);
  var
    a,b:Integer;
    sReal,sImag,cc,ss:Single;
  begin
    Theta:=Theta/(NumSamples);
    for a:=0 to NumSamples-1 do begin
      sReal:=0;
      sImag:=0;
      for b:=0 to NumSamples-1 do begin
        cc:=Cos(Theta*a*b);
        ss:=Sin(Theta*a*b);
        sReal:=sReal+RealIn[b]*cc-ImagIn[b]*ss;
        sReal:=sReal+RealIn[b]*ss+ImagIn[b]*cc;
      end;
      RealOut[a]:=sReal;
      ImagOut[a]:=sImag;
    end;
  end;

  procedure MakeRidgelet(var w:TWaveLetData);       //calcule une transforme en ridgelets
  var
    a,b:Integer;
    wReal,wImag,v:TWaveLetData;
    tReal,tImag,t:array[0..2*MaxWaveletSize-1] of Single;
  begin
    ZeroMemory(@v,SizeOf(v));
    for a:=0 to FWaveletSize-1 do
      FourierTransform(w[a],v[a],wReal[a],wImag[a],2*pi,FWaveletSize);
    for a:=0 to FWaveletSize-1 do begin
      for b:=0 to FWaveletSize-1 do begin
        tReal[b]:=wReal[b,a];
        tImag[b]:=wImag[b,a];
      end;
      FourierTransform(tReal,tImag,tReal,tImag,2*pi,FWaveletSize);
      for b:=0 to FWaveletSize-1 do begin
        wReal[b,a]:=tReal[b];
        wImag[b,a]:=tImag[b];
      end;
    end;
    v:=wReal;
    w:=wImag;
{    for a:=0 to FWaveletSize-1 do
      for b:=0 to FWaveletSize-1 do begin
        wReal[(a+((FWaveletSize-2) div 2)) mod FWaveletSize,(b+((FWaveletSize-2) div 2)) mod FWaveletSize]:=v[a,b];
        wImag[(a+((FWaveletSize-2) div 2)) mod FWaveletSize,(b+((FWaveletSize-2) div 2)) mod FWaveletSize]:=w[a,b];
      end;}
    for a:=0 to FWaveletSize-1 do begin
      for b:=0 to FWaveletSize-1 do begin                                      //angles entre -pi/4 et pi/4
        tReal[b]:=wReal[b,Round(a*b/(FWaveletSize-1))];
        tImag[b]:=wImag[b,Round(a*b/(FWaveletSize-1))];
      end;
      FourierTransform(tReal,tImag,tReal,tImag,-2*pi,FWaveletSize);
      for b:=0 to FWaveletSize-1 do
        w[a,b]:=Sqrt(Sqr(tReal[a])+Sqr(tImag[a]))/Sqr(FWaveletSize);
      RecMake1DWaveLet(w[a],FWaveletSize);
      for b:=0 to FWaveletSize-1 do begin                                      //angles entre -pi/4 et pi/4
        tReal[b]:=wReal[Round(a*(FWaveletSize-1-b)/(FWaveletSize-1)),b];
        tImag[b]:=wImag[Round(a*(FWaveletSize-1-b)/(FWaveletSize-1)),b];
      end;
      FourierTransform(tReal,tImag,tReal,tImag,-2*pi,FWaveletSize);
      for b:=0 to FWaveletSize-1 do
        w[a+FWaveletSize,b]:=Sqrt(Sqr(tReal[a])+Sqr(tImag[a]))/Sqr(FWaveletSize);
      RecMake1DWaveLet(w[a+FWaveletSize],FWaveletSize);
{      for b:=0 to FWaveletSize-1 do begin                                      //angles entre pi/4 et 3*pi/4
        tReal[(b+((FWaveletSize+2) div 2)) mod FWaveletSize]:=wReal[a+Round((FWaveletSize-1-2*a)*b/(FWaveletSize-1)),FWaveletSize-1-b];
        tImag[(b+((FWaveletSize+2) div 2)) mod FWaveletSize]:=wImag[a+Round((FWaveletSize-1-2*a)*b/(FWaveletSize-1)),FWaveletSize-1-b];
      end;
      FourierTransform(tReal,tImag,tReal,tImag,-2*pi,FWaveletSize);
      for b:=0 to FWaveletSize-1 do
        w[a+FWaveletSize,b]:=Sqrt(Sqr(tReal[a])+Sqr(tImag[a]))/FWaveletSize;
      RecMake1DWaveLet(w[a+FWaveletSize],FWaveletSize);}
    end;
    for a:=0 to FWaveletSize-1 do begin        //nouvelle transforme en ondelettes unidimensionnelle mais orthogonale cette fois
      for b:=0 to 2*FWaveletSize-1 do
        t[b]:=w[b,a];
      RecMake1DWaveLet(t,2*FWaveletSize);
      for b:=0 to FWaveletSize-1 do            //on ne gardera pas les plus hautes frquences, de faon  ce qu'on n'ait plus que des donnes sur un carr au final, et non un rectangle
        w[b,a]:=t[b];
    end;
  end;

var
  a,b:Integer;
begin
  if not Inverted then
    FRasterBitmap.Canvas.Brush.Color:=0          //on s'occupe ici des bords ventuels de l'image (il se peut que le rectangle soit en dehors de l'image)
  else
    FRasterBitmap.Canvas.Brush.Color:=clWhite;
  FRasterBitmap.Canvas.Rectangle(0,0,FRasterBitmap.Width,FRasterBitmap.Height);   //on recouvre les bords  l'extrieur de l'image par du blanc ou du noir selon qu'on veut la transforme de l'image inverse ou non
  if x+w>p.Width then   //on rajuste le rectangle au cas o il tombe  l'extrieur
    w:=p.Width-x;
  if y+h>p.Height then
    h:=p.Height-y;
  //on recopie le bitmap source de faon  le faire tenir dans un carr de la taille de la fentre d'ondelettes:
  if Inverted then
    StretchBlt(FRasterBitmap.Canvas.Handle,0,0,FWaveletSize,FWaveletSize,p.Canvas.Handle,x,y,w,h,SRCCOPY)       //cas invers
  else
    StretchBlt(FRasterBitmap.Canvas.Handle,0,0,FWaveletSize,FWaveletSize,p.Canvas.Handle,x,y,w,h,NOTSRCCOPY);   //cas non invers
  for a:=0 to FWaveletSize-1 do
    for b:=0 to FWaveletSize-1 do
      Result[a,b]:=PByteArray(FRasterBitmap.ScanLine[a])[b];     //on rcupre les coefficients initiaux
  if not RidgeletModeCheckBox.Checked then
    RecMake2DWavelet(Result,FWaveletSize)              //et on calcule l'ondelette
  else
    MakeRidgelet(Result);                              //ou la transforme en ridgelets
end;

procedure TMainForm.ProcessASCIIConversion;             //conversion de l'image filtre en ASCII
var
  g:PGlyphData;
  x,y,i:Integer;
  c,d:Char;
  r,s,rr1,rr2,rr3:Single;
  u:string;
  l:TList;
  Aborted:Boolean;

  function WaveletErrorMeasure(w1,w2:TWaveletData):Single;   //calcul de la distance de 2 ondelettes en fonction de la matrice courante de pondration
  var
    a,b,s:Integer;
    r1,r2,r3:Single;
    m:TWeightMatrix;
  begin
    s:=1;
    r1:=rr1;           //constantes initiales de renormalisation
    r2:=rr2;
    r3:=rr3;
    Result:=0;
    while s<FWaveletSize do begin
      if s=1 then
        Result:=Abs(w1[0,0]-w2[0,0])*FWeightMatrix[0,0]
      else begin
        ZeroMemory(@m,SizeOf(m));
        for a:=0 to s-1 do                 //on calcule la norme par "paquets" adapts  l'ondelette
          for b:=0 to s-1 do begin
            m[1,0]:=m[1,0]+(w1[a+s,b]-w2[a+s,b]);
            m[0,1]:=m[0,1]+(w1[a,b+s]-w2[a,b+s]);
            m[1,1]:=m[1,1]+(w1[a+s,b+s]-w2[a+s,b+s]);
          end;
        Result:=Result+Abs(m[1,0])*FWeightMatrix[1,0]*r1+Abs(m[0,1])*FWeightMatrix[0,1]*r2+Abs(m[1,1])*FWeightMatrix[1,1]*r3;
        r1:=r1*FWeightMatrix[1,0]*0.25;
        r2:=r2*FWeightMatrix[0,1]*0.25;
        r3:=r3*FWeightMatrix[1,1]*0.25;
      end;
      s:=2*s;
    end;
  end;

  function LogStep(x:Single):Single;  //calcul d'une chelle logarithmique en base 10
  const
    Theta=10;
  begin
    Result:=(Exp(Ln(Theta)*x)-1)/(Theta-1);
  end;

  procedure AdjustMatrix;        //cette fonction tente de calculer automatiquement la matrice de pondration la mieux adapte
  var
    c:Char;
    a:Integer;
    v,w:TWeightMatrix;
  begin
    a:=0;
    ZeroMemory(@w,SizeOf(w));
    for c:=#30 to #255 do begin           //pour commencer on fait la moyenne des pouvoirs discriminants des caractres dans les diffrentes directions
      with FGlyphsData[c] do begin
        if (Width=0) or (OnlyAlphanumericCheckBox.Checked and not (IsCharAlphaNumeric(c))) then  // on limine les caractres interdits
          Continue;
        Inc(a);
        w[0,0]:=w[0,0]+WeightsDomain[0,0];
        w[1,0]:=w[1,0]+WeightsDomain[1,0];
        w[0,1]:=w[0,1]+WeightsDomain[0,1];
        w[1,1]:=w[1,1]+WeightsDomain[1,1];
      end;
    end;
    w[0,0]:=w[0,0]/a;
    w[1,0]:=w[1,0]/a;
    w[0,1]:=w[0,1]/a;
    w[1,1]:=w[1,1]/a;
    ZeroMemory(@v,SizeOf(v));
    for c:=#30 to #255 do begin                  //ensuite on calcule l'cart-type
      with FGlyphsData[c] do begin
        if (Width=0) or (OnlyAlphanumericCheckBox.Checked and not (IsCharAlphaNumeric(c))) then  // on limine les caractres interdits
          Continue;
        v[0,0]:=Sqr(w[0,0]-WeightsDomain[0,0]);
        v[1,0]:=Sqr(w[1,0]-WeightsDomain[1,0]);
        v[0,1]:=Sqr(w[0,1]-WeightsDomain[0,1]);
        v[1,1]:=Sqr(w[1,1]-WeightsDomain[1,1]);
      end;
    end;
    v[0,0]:=Sqrt(v[0,0]/a);
    v[1,0]:=Sqrt(v[1,0]/a);
    v[0,1]:=Sqrt(v[0,1]/a);
    v[1,1]:=Sqrt(v[1,1]/a);
    SymmetricCheckBox.Checked:=False;   //dans le cas de l'auto-ajustement, la matrice ne sera pas forcment symtrique
    MatrixTrackBar1.Position:=1000;              //on utilise la valeur de l'cart-type pour dterminer les coefficients de la matrice.
    MatrixTrackBar2.Position:=Round(100*v[1,0]); //Plus l'cart type est grand pour une police donne dans une direction, plus elle est susceptible
    MatrixTrackBar3.Position:=Round(100*v[0,1]); //de pouvoir bien discriminer des paterns diffrents dans cette direction-l. Une police dont tous les
    MatrixTrackBar4.Position:=Round(50*v[1,1]);  //caractres seraient identiques par exemple aurait une matrice de pondration nulle dans les 3 directions.
  end;

  function Renormalize(x:Single):Single;
  begin
    if x=0 then
      Result:=0
    else
      Result:=(1-0.25*x)/(1-Exp(Ln(0.25*x)*Ln(FWaveLetSize)/Ln(2)));
  end;

begin
  ProgressForm.InitProgress('Converting to ASCII...',Aborted);   //barre de progression
  FRasterBitmap.Width:=FWaveLetSize;          //on ajuste les dimensions du bitmap de travail
  FRasterBitmap.Height:=FWaveLetSize;
  ASCIIMemo.Font.Assign(FontDialog.Font);           //on rcupre la police choisie par l'utilisateur
  if WhiteOnBlackCheckBox.Checked then begin        //white on black / black on white
    ASCIIMemo.Font.Color:=clWhite;
    ASCIIMemo.Color:=0;
  end else begin
    ASCIIMemo.Font.Color:=0;
    ASCIIMemo.Color:=clWhite;
  end;
  MakeFontGlyphs;         //on calcule les donnes des caractres de la police
  ASCIIMemo.Clear;        //on efface
  x:=0;
  y:=0;
  u:='';
  if AutoAdjustCheckBox.Checked then      //auto-ajustement des coefficients de la matrice si ncessaire
    AdjustMatrix;
  FWeightMatrix[0,0]:=LogStep(MatrixTrackBar1.Position/1000);     //on rcupre la matrice de pondration courante en fonction des trackbars, en chelle logarithmique
  FWeightMatrix[1,0]:=LogStep(MatrixTrackBar2.Position/1000);
  FWeightMatrix[0,1]:=LogStep(MatrixTrackBar3.Position/1000);
  FWeightMatrix[1,1]:=LogStep(MatrixTrackBar4.Position/1000);
  rr1:=Renormalize(FWeightMatrix[1,0]);   //coefficients de renormalisation utiliss plus tard par ErrorMeasure
  rr2:=Renormalize(FWeightMatrix[0,1]);
  rr3:=Renormalize(FWeightMatrix[1,1]);
  l:=TList.Create;  //Liste des tailles de rectangle dont on a dj calcul la transforme d'ondelettes (pour viter d'avoir  les recalculer). Pour les polices
  while y<FFilteredBitmap.Height do begin   // de taille variable, on peut avoir  calculer la trasnforme sur des rectangles de taille diffrente
    g:=nil;
    s:=1E10;
    d:=#0;
    for c:=#30 to #255 do begin
      with FGlyphsData[c] do begin
        if (Width=0) or (OnlyAlphanumericCheckBox.Checked and not (IsCharAlphaNumeric(c))) then  //on limine les caractres interdits
          Continue;
        g:=nil;
        for i:=0 to l.Count-1 do                                                                 //on cherche dans la liste si on a dj calcul la transforme pour la taille du caractre
          if (Width=PGlyphData(l[i]).Width) and (Height=PGlyphData(l[i]).Height) then begin
            g:=PGlyphData(l[i]);
            Break;
          end;
        if not Assigned(g) then begin      //si la taille n'est pas trouve dans la liste, on effectue la transforme
          New(g);
          g.Width:=Width;
          g.Height:=Height;
          g.WaveletData:=MakeWaveletData(FFilteredBitmap,X,Y,Width,Height,WhiteOnBlackCheckBox.Checked);  //prise en compte du mode white on black
          l.Add(g);      //on ajoutte  la liste la transforme calcule
        end;
      end;
      r:=WaveletErrorMeasure(g.WaveletData,FGlyphsData[c].WaveletData)*g.Width*g.Height;  //on pondre l'erreur de mesure par la surface du caractre
//      r:=WaveLetErrorMeasure(g.WaveletData,FGlyphsData[c].WaveletData)*g.Width*g.Height;  //on pondre l'erreur de mesure par la surface du caractre
      if r<s then begin     //on garde dans d le caractre qui a la plus petite erreur de mesure
        d:=c;
        s:=r;
      end;
    end;
    for i:=0 to l.Count-1 do            //on libre la mmoire
      FreeMem(l[i]);
    l.Clear;
    u:=u+d;
    x:=x+FGlyphsData[d].Width;
    if x>FFilteredBitmap.Width then begin  //passage  la ligne
      ProgressForm.UpdateProgress(Round(100*y/FFilteredBitmap.Height));    //mise  jour de la progression
      if Aborted then              //teste si l'utilisateur a cliqu sur "cancel"
        Break;
      x:=0;
      y:=y+FGlyphsData[d].Height;
      ASCIIMemo.Lines.Add(u);     //on ajoutte la ligne de caractres
      u:='';
    end;
  end;
  l.Destroy;                    //on libre la mmoire
  ProgressForm.CloseProgress;   //on ferme la fentre de progression
end;

procedure TMainForm.ChangeFontSpeedButtonClick(Sender: TObject);
begin
  if FontDialog.Execute then      //changement de police
    ApplyFont;     //mise  jour de l'interface
end;

procedure TMainForm.Applyfont;    //mise  jour de l'interface pour le nom et les caractristiques de la police
begin
  FontDialog.Font.Color:=0;
  FontPanel.Caption:=' Font: '+FontDialog.Font.Name+', '+IntToStr(FontDialog.Font.Size)+' pts';
  if fsUnderline in FontDialog.Font.Style then
    FontPanel.Caption:=FontPanel.Caption+', underline';
  if fsStrikeout in FontDialog.Font.Style then
    FontPanel.Caption:=FontPanel.Caption+', strike-out';
  if fsBold in FontDialog.Font.Style then
    FontPanel.Caption:=FontPanel.Caption+', bold';
  if fsItalic in FontDialog.Font.Style then
    FontPanel.Caption:=FontPanel.Caption+', italic';
end;

procedure TMainForm.CopyToClipboardSpeedButtonClick(Sender: TObject);
begin
  Clipboard.AsText:=ASCIIMemo.Text;    //on copie le texte dans le presse-papier
end;

procedure TMainForm.CopyHTMLToClipboardSpeedButtonClick(Sender: TObject);    //gnration du code HTML dans le presse-papier
var
  a,b:Integer;
  s,t:string;
begin
  t:='<DIV style="font-family:'+FontDialog.Font.Name;
  if WhiteOnBlackCheckBox.Checked then
    t:=t+'; color:#FFFFFF; font-size:'+IntToStr(FontDialog.Font.Size)+'; background-color:#000000;'
  else
    t:=t+'; color:#000000; font-size:'+IntToStr(FontDialog.Font.Size)+'; background-color:#FFFFFF;';
  if fsUnderline in FontDialog.Font.Style then
    t:=t+' text-decoration:underline;';
  if fsStrikeout in FontDialog.Font.Style then
    t:=t+' text-decoration:line-through;';
  if fsBold in FontDialog.Font.Style then
    t:=t+' font-weight:bold;';
  if fsItalic in FontDialog.Font.Style then
    t:=t+' font-style:italic;';
  t:=t+'">';
  for a:=0 to ASCIIMemo.Lines.Count do begin
    s:=ASCIIMemo.Lines[a];
    for b:=1 to Length(s) do
      if s[b]=#32 then
        t:=t+'&nbsp'
      else
        t:=t+'&#'+IntToStr(Ord(s[b]));
    t:=t+'<BR>';
  end;
  t:=t+'<DIV>';
  Clipboard.AsText:=t;
end;

procedure TMainForm.MatrixTrackBar1Change(Sender: TObject);    //si l'utilisateur change la position de l'une des 4 trackbars de la matrice de pondration
begin
  if (Sender=MatrixTrackBar2) and SymmetricCheckBox.Checked then    //si on est en mode symmtrie: les coefficients diagonaux doivent tre gaux
    MatrixTrackBar3.Position:=MatrixTrackBar2.Position;
  if (Sender=MatrixTrackBar3) and SymmetricCheckBox.Checked then
    MatrixTrackBar2.Position:=MatrixTrackBar3.Position;
  with TTrackBar(Sender) do
    TPanel(Tag).Caption:=Hint+': '+IntToStr(Position);    //Tag contient le panel qui sert  afficher la valeur du coefficient, et Hint le nom du coefficient
end;

procedure TMainForm.MatrixListBoxClick(Sender: TObject);      //clic sur la listbox des matrices disponibles
begin
  DeleteMatrixSpeedButton.Enabled:=MatrixListBox.ItemIndex>-1;    //mise  jour de l'interface
  LoadMatrixSpeedButton.Enabled:=MatrixListBox.ItemIndex>-1;
  if MatrixListBox.ItemIndex>-1 then
    MatrixNameEdit.Text:=MatrixListBox.Items[MatrixListBox.ItemIndex];
end;

procedure TMainForm.MatrixNameEditChange(Sender: TObject);
begin
  SaveNewMatrixSpeedButton.Enabled:=MatrixNameEdit.Text<>'';    //mise  jour de l'interface
end;

procedure TMainForm.IniLoad;     //lecture des informations du fichier ini
var
  f:TIniFile;
  a:Integer;
  p:PWeightMatrix;
  m:TMemoryStream;
begin
  f:=TIniFile.Create(ChangeFIleExt(Application.ExeName,'.ini'));   //le fichier ini aura le mme nom que l'excutable
  m:=TMemoryStream.Create;
  f.ReadSection('MatrixList',MatrixListBox.Items);
  for a:=0 to MatrixListBox.Items.Count-1 do begin           //on lit toutes les matrices sauvegardes
    m.Clear;
    f.ReadBinaryStream('MatrixList',MatrixListBox.Items[a],m);
    m.Position:=0;
    New(p);
    m.Read(p^,SizeOf(TWeightMatrix));
    MatrixListBox.Items.Objects[a]:=TObject(p);   //on ajoutte les matrices de pondrations dans les Objects[.] de la ListBox
  end;
  m.Destroy;
  f.Destroy;
end;

procedure TMainForm.IniSave;       //sauvegarde du fichier ini
var
  f:TIniFile;
  a:Integer;
  m:TMemoryStream;
begin
  f:=TIniFile.Create(ChangeFIleExt(Application.ExeName,'.ini'));
  f.EraseSection('MatrixList');
  m:=TMemoryStream.Create;
  for a:=MatrixListBox.Items.Count-1 downto 0 do begin
    m.Clear;
    m.Write(PWeightMatrix(MatrixListBox.Items.Objects[a])^,SizeOf(TWeightMatrix));    //on crit toutes les matrices dans le fichier ini
    m.Position:=0;
    f.WriteBinaryStream('MatrixList',MatrixListBox.Items[a],m);
    FreeMem(Pointer(MatrixListBox.Items.Objects[a]));
  end;
  m.Destroy;
  f.Destroy;
end;

procedure TMainForm.MatrixListBoxDblClick(Sender: TObject);       //double-clic sur la liste des matrices prdfinies
var
  m:TWeightMatrix;
begin
  MatrixNameEdit.Text:=MatrixListBox.Items[MatrixListBox.ItemIndex];      //mise  jour de l'interface
  m:=PWeightMatrix(MatrixListBox.Items.Objects[MatrixListBox.ItemIndex])^;
  MatrixTrackBar1.Position:=Round(m[0,0]);
  MatrixTrackBar2.Position:=Round(m[1,0]);
  MatrixTrackBar3.Position:=Round(m[0,1]);
  MatrixTrackBar4.Position:=Round(m[1,1]);
end;

procedure TMainForm.SaveNewMatrixSpeedButtonClick(Sender: TObject);       //enregistrement d'une nouvelle matrice dans la liste
var
  a:Integer;
  p:PWeightMatrix;
begin
  a:=MatrixListBox.Items.IndexOf(MatrixNameEdit.Text);
  if (a<>-1) and (MessageDlg('Overwrite "'+MatrixNameEdit.Text+'"?',mtConfirmation,[mbYes,mbNo],0)=mrNo) then  //confirmation d'crasement
    Exit;
  if a=-1 then begin
    New(p);
    MatrixListBox.Items.AddObject(MatrixNameEdit.Text,TObject(p));
  end else
    p:=PWeightMatrix(MatrixListBox.Items.Objects[a]);
  p^[0,0]:=MatrixTrackBar1.Position;       //lecture des coefficients
  p^[1,0]:=MatrixTrackBar2.Position;
  p^[0,1]:=MatrixTrackBar3.Position;
  p^[1,1]:=MatrixTrackBar4.Position;
end;

procedure TMainForm.DeleteMatrixSpeedButtonClick(Sender: TObject);        //suppression d'une matrice de la liste
begin
  FreeMem(Pointer(MatrixListBox.Items.Objects[MatrixListBox.ItemIndex]));
  MatrixListBox.Items.Delete(MatrixListBox.ItemIndex);
  MatrixListBoxClick(nil);
end;

procedure TMainForm.FilteredImagePaintBoxPaint(Sender: TObject);     //affichage de l'image filtre
begin
  FilteredImagePaintBox.Canvas.Draw(0,0,FFilteredBitmap);
end;

procedure TMainForm.BuildFilteredBitmap;       //construction de l'image filtre
var
  a,b,c,i,j,l,w,h,x,y,FilterWidth:Integer;
  t:array of Byte;
  Aborted:Boolean;
  p:TBitmap;
  a1,a2,a3,a4,xx,yy:Single;

  function Clamp(x:Integer;Min,Max:Integer):Integer;     //pour qu'un nombre reste dans un intervalle donn
  begin
    Result:=x;
    if x<Min then
      Result:=Min;
    if x>Max then
      Result:=Max;
  end;

  procedure Swap(a,b:Integer);         //inversion de 2 lments du tableau
  var
    s:Byte;
  begin
    s:=t[a];
    t[a]:=t[b];
    t[b]:=s;
  end;

  procedure SortArray(a,b:Integer);    //procdure rcursive de tri "quicksort"
  var
    c,d:Integer;
  begin
    case b-a of
      0:;
      1:begin
        if t[a]>t[b] then
          Swap(a,b);
      end;
    else
      c:=(a+b) div 2;
      d:=a;
      while d<c do begin
        if t[d]>t[c] then begin
          Swap(c,d);
          Dec(c);
          Swap(c,d);
        end else
          Inc(d);
      end;
      d:=b;
      while d>c do begin
        if t[d]<t[c] then begin
          Swap(c,d);
          Inc(c);
          Swap(c,d);
        end else
          Dec(d);
      end;
      if c>a then
        SortArray(a,c-1);
      if b>c then
        SortArray(c+1,b);
    end;
  end;

  function GetOriginalIntensity(x,y:Integer):Single;  //calcul de l'intensit d'une couleur en RGB
  var
    p:PByteArray;
  begin
    p:=PByteArray(FOriginalBitmap.ScanLine[x]);
    Result:=Sqrt(Sqr(Integer(p^[3*y]))+Sqr(Integer(p^[3*y+1]))+Sqr(Integer(p^[3*y+2])))/Sqrt(3);
  end;

  function BuildBilinearValues:Byte;  //Interpolation bilinaire
  var
    u,v:Single;
  begin
    a1:=GetOriginalIntensity(x,y);
    a2:=GetOriginalIntensity(x+1,y);
    a3:=GetOriginalIntensity(x,y+1);
    a4:=GetOriginalIntensity(x+1,y+1);
    u:=xx-x;
    v:=yy-y;
    Result:=Round(a1+(a2-a1)*u+(a3-a1)*v+(a4-a3-a2+a1)*u*v);
  end;

begin
  ProgressForm.InitProgress('Filtering source image...',Aborted);    //Affichage de la fentre de progression
  FilterWidth:=MedianFilterSizeRadioGroup.ItemIndex*2+1;             //Taille du filtre mdian
  w:=Round(FOriginalBitmap.Width*XFactorTrackBar.Position/100);      //Calcul de la nouvelle taille d'image
  h:=Round(FOriginalBitmap.Height*YFactorTrackBar.Position/100);
  FFilteredBitmap.Width:=w;
  FFilteredBitmap.Height:=h;
  case ResizingMethodRadioGroup.ItemIndex of         //les 2 mthodes pour redimensionner
    0:begin
      StretchBlt(FFilteredBitmap.Canvas.Handle,0,0,w,h,FOriginalBitmap.Canvas.Handle,0,0,FOriginalBitmap.Width,FOriginalBitmap.Height,SRCCOPY); //mthode de base
    end;
    1:begin                //interpolation bilinaire
      FOriginalBitmap.PixelFormat:=pf24bit;
      for a:=0 to h-1 do begin
        xx:=a*(FOriginalBitmap.Height-1)/(h-1);
        x:=Ceil(xx)-1;
        if x<0 then
          x:=0;
        for b:=0 to w-1 do begin
          yy:=b*(FOriginalBitmap.Width-1)/(w-1);
          y:=Ceil(yy)-1;
          if y<0 then
            y:=0;
          PByteArray(FFilteredBitmap.ScanLine[a])^[b]:=BuildBilinearValues;
        end;
      end;
    end;
  end;
  SetLength(t,FilterWidth*FilterWidth);     //filtre mdian: son intrt est de "lisser" les contours de l'image et d'enlever du bruit.
  l:=FilterWidth div 2;
  p:=TBitmap.Create;
  p.Assign(FFilteredBitmap);
  for a:=0 to h-1 do begin
    ProgressForm.UpdateProgress(Round(100*a/h));  //mise  jour de la barre de progression
    if Aborted then   //si l'utilisateur annule, on quitte
      Break;
    for b:=0 to w-1 do begin
      c:=0;
      for i:=-l to l do
        for j:=-l to l do begin
          t[c]:=PByteArray(FFilteredBitmap.ScanLine[Clamp(a+i,0,h-1)])^[Clamp(b+j,0,w-1)];   //calcul des valeurs des n pixels voisins de (a,b)
          Inc(c);
        end;
      SortArray(0,High(t));          //on trie ces valeurs par ordre croissant
      PByteArray(p.ScanLine[a])^[b]:=Round(t[(FilterWidth*FilterWidth) div 2]);  //on garde la valeur mdiane (celle du milieu dans l'ordre croissant)
    end;
  end;
  if not Aborted then    //si l'utilisateur a annul, on ne change pas le bitmap flitr
    FFilteredBitmap.Assign(p);
  p.Destroy;          //libration de la mmoire
  SetLength(t,0);
  FilteredImagePaintBox.SetBounds(0,0,w,h);      //affichage de l'image calcule
  FilteredImagePaintBox.Repaint;
  ProgressForm.CloseProgress;        //fermeture de la barre de progression
end;

procedure TMainForm.FontDialogApply(Sender: TObject; Wnd: HWND);    //lorsque l'on clique sur le boutton "Apply" de la boite de dialogue de choix de police
begin
  Applyfont;         //mise  jour de l'interface
  ProcessASCIIConversion;    //on convertit
  Enabled:=False;            //on se remet en mode "modal"
  BringWindowToTop(FontDialog.Handle);    //on remet la boite de dialogue en haut de toutes les fentres
end;

procedure TMainForm.WaveletQualityRadioGroupClick(Sender: TObject);
begin
  FWaveletSize:=Round(Exp(Ln(2)*(WaveletQualityRadioGroup.ItemIndex+1)));   //calcul de la taille de l'ondelette en fonction de la qualit
end;

procedure TMainForm.PageControlChange(Sender: TObject);     //lorsqu'on change d'onglet
begin
  Caption:='Graphic to ASCII converter (step '+IntToStr(PageControl.TabIndex+1)+' of 3)';   //mise  jour du numro de l'tape
  BackBitBtn.Enabled:=PageControl.TabIndex>0;     //bouttons "suivant" et "prcdant"
  NextBitBtn.Enabled:=PageControl.TabIndex<2;
  FilteredImageTabSheet.TabVisible:=not FOriginalBitmap.Empty;    //mise  jour de l'interface: on ne peut passer  l'tape suivante que si l'tape d'avant a t effectue
  ASCIIConversionTabSheet.TabVisible:=not FFilteredBitmap.Empty;
  case PageControl.TabIndex of
    0:ProcessBitBtn.Caption:='Load image...';
    1:ProcessBitBtn.Caption:='Build filtered image';
    2:ProcessBitBtn.Caption:='Convert to ASCII';
  end;
end;

procedure TMainForm.NextBitBtnClick(Sender: TObject);        //Suivant
begin
  if PageControl.TabIndex<2 then begin
    if not PageControl.Pages[PageControl.TabIndex+1].TabVisible then
      ProcessBitBtnClick(nil);
    PageControl.TabIndex:=PageControl.TabIndex+1;
  end;
  PageControlChange(nil);
end;

procedure TMainForm.BackBitBtnClick(Sender: TObject);         //prcdant
begin
  if PageControl.TabIndex>0 then
    PageControl.TabIndex:=PageControl.TabIndex-1;
  PageControlChange(nil);
end;

procedure TMainForm.ProcessBitBtnClick(Sender: TObject);     //procder  l'tape courante
begin
  case PageControl.TabIndex of      //en fonction de l'tape o l'on en est:
    0:LoadImage;                  // 0 : on charge une image
    1:BuildFilteredBitmap;        // 1 : on construit l'image filtre
    2:ProcessASCIIConversion;     // 2 : on convertit en ASCII
  end;
  PageControlChange(nil);        //mise  jour de l'interface
end;

procedure TMainForm.LoadImage;      //chargement de l'image initiale
var
  p:TPicture;
begin
  if OpenPictureDialog.Execute then begin
    p:=TPicture.Create;
    p.LoadFromFile(OpenPictureDialog.FileName);
    FOriginalBitmap.Assign(p.Graphic);
    p.Destroy;
    OriginalImagePaintBox.SetBounds(0,0,FOriginalBitmap.Width,FOriginalBitmap.Height);
    OriginalImagePaintBox.Repaint;
    FFilteredBitmap.Destroy;            //on dtruit l'image filtre et on en recre une vide (pour qu'on ne puisse pas passer  l'tape 3)
    FFilteredBitmap:=TBitmap.Create;
    FFilteredBitmap.PixelFormat:=pf8bit;
    FFilteredBitmap.Palette:=CreatePalette(FGrayScaleLogPalette^);
    ApplyResize;       //mise  jour de l'interface
  end;
end;

procedure TMainForm.XFactorTrackBarChange(Sender: TObject);       //Trackbars de zoom
begin
  if KeepRatioCheckBox.Checked then begin
    if Sender=XFactorTrackBar then
      YFactorTrackBar.Position:=XFactorTrackBar.Position;
    if Sender=YFactorTrackBar then
      XFactorTrackBar.Position:=YFactorTrackBar.Position;
  end;
  ApplyResize;    //mise  jour de l'interface
end;

procedure TMainForm.ApplyResize;       //pour afficher les informations de redimensionnement sur le panel correspondant
var
  w,h:Integer;
begin
  w:=FOriginalBitmap.Width;
  h:=FOriginalBitmap.Height;
  ResizePanel.Caption:=IntToStr(w)+' x '+IntToStr(h)+' -> '+IntToStr(Round(w*XFactorTrackBar.Position/100))+' x '+IntToStr(Round(h*YFactorTrackBar.Position/100));
end;

procedure TMainForm.RatioSpeedButton1Click(Sender: TObject);      //ratios prdfinis (voir la proprit Tag des speedbuttons concerns)
begin
  with TSpeedButton(Sender) do begin
    XFactorTrackBar.Position:=Tag*25;
    YFactorTrackBar.Position:=Tag*25;
  end;
end;

procedure TMainForm.AutoAdjustCheckBoxClick(Sender: TObject);      //Si on est en mode auto-ajustement de la matrice, les trackbars
begin                                                              //associes passent en lecture seule.
  MatrixTrackBar1.Enabled:=not AutoAdjustCheckBox.Checked;
  MatrixTrackBar2.Enabled:=not AutoAdjustCheckBox.Checked;
  MatrixTrackBar3.Enabled:=not AutoAdjustCheckBox.Checked;
  MatrixTrackBar4.Enabled:=not AutoAdjustCheckBox.Checked;
end;

end.
